<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps;

use TypeError;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * A typed array (index-based array).
 *
 * @phpstan-extends \ArrayObject<int|string, mixed>
 */
class TypedArray extends \ArrayObject {
	/**
	 * The type.
	 *
	 * @var string
	 */
	protected string $type;

	/**
	 * Constructor.
	 *
	 * @param array<mixed> ...$args An arbitrary number of arguments.
	 */
	protected function __construct( ...$args ) {
		parent::__construct( ...$args );
	}

	/**
	 * Creates an instance of this class for a specific type.
	 *
	 * @param string $type The type of the array elements.
	 */
	public static function for_type( string $type ): self {
		$ret       = new self();
		$ret->type = $type;
		return $ret;
	}

	/**
	 * Adds a value to the array.
	 *
	 * @param int|string $key The key to add.
	 * @param mixed      $value The new value to add.
	 * @throws \TypeError If the value is not of the correct type.
	 */
	public function offsetSet( mixed $key, mixed $value ): void {
		if ( ! ( $value instanceof $this->type ) ) {
			throw new \TypeError( sprintf( 'Only values of type %s are supported', $this->type ) ); // phpcs:ignore
		}

		parent::offsetSet( $key, $value );
	}
}
